## This code is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 2 of the License, or
## (at your option) any later version.
## 
## For a copy of the GNU General Public License, 
## see <http://www.gnu.org/licenses/>.

## (c) 2021 Sarah Humberg




# .................................................... ----
# DESCRIPTION                                          ----

# This code is a background code that accompanies the preregistration for the following study: 
# Title: Does Grateful Love Take Two? New insights into Partner Match on Gratitude Using Seven Longitudinal Datasets
# Authors: Yoobin Park, Sarah Humberg, Amie M. Gordon, Amy Muise, Emily A. Impett

# This part of the code defines functions to conduct dyadic RSA for indistinguishable dyads.




# .................................................... ----
# PREPARATIONS                                         ----

## load packages ----

if(!require(lavaan)){install.packages("lavaan")}; library(lavaan) # model estimation

if(!require(fastDummies)){install.packages("fastDummies")}; library(fastDummies) # creation of dummy variables




# .................................................... ----
# FUNCTION FOR PRE-ANALYSIS: DISCREPANCY OF PREDICTORS ----


# define function to count predictor combinations that are considered congruent (within half a grand standard deviation) 
# vs. discrepant in the one or the other direction
# (version for long format data frame)
discr.long <- function(data, couple.identifier, x){
  
  df <- data
  
  df$x.std <- df[,x]/sd(df[,x], na.rm=T)
  
  agg <- aggregate(eval(parse(text = paste0("x.std ~ ", couple.identifier))), data=df, FUN = function(x){abs(x[1]-x[2])} )
  
  cong <- cut(agg$x.std, breaks = c(0, .5, Inf), labels = c("Congruence", "Incongruence"))

  tab <- as.data.frame(round(prop.table(table(cong)), 2)*100)
  
  return(tab)
  
}



# .................................................... ----
# FUNCTION FOR DYADIC RSA                              ----

drsa <- function(data, 
                 predictor, 
                 outcome, 
                 couple.identifier,
                 
                 control.personvars = c(), # control variables that do vary between the two persons within a couple (e.g., age)
                 control.couplevars = c(), # control variables that do not vary between the two persons within a couple (e.g., sample dummy, relationship length)
                 control.redundant = c(), # control variables that do vary between the persons but are perfectly correlated (only relevant for us if we run the analyses with control for gender in a sample with only heterosexual couples)
                 
                 rm.influential = F,
                 
                 missing = "fiml", 
                 
                 se = "boot", 
                 bootstrap = 10000, 
                 seed = 1337
){
  

  # prepare data
  
    # reduce data frame to the variables that will be used 
    
    datlong <- data[, c(couple.identifier, predictor, outcome, control.personvars, control.couplevars, control.redundant)]
    
    
    # randomly decide for each couple who will be treated as "Partner 1" and who will be "Partner 2"
    
    datlong$partner <- rep(0, nrow(datlong))
    
    set.seed(seed)
    
    for(c in unique(datlong[,couple.identifier]) ){
      datlong[datlong[,couple.identifier] == c, "partner"] <- sample(1:2)
    }

    
    # restructure the data to wide format
    
    dat <- reshape(datlong, 
                   v.names = c(predictor, outcome, control.personvars, control.redundant), 
                   timevar = "partner", idvar = "coupleID", 
                   direction = "wide")
    
    
    # extract variable names: x1 = predictor of partner 1, x2 = pred of partner 2, z1 / z2 = outcome of partner 1 / 2
    x1 <- paste0(predictor, ".1")
    x2 <- paste0(predictor, ".2")
    z1 <- paste0(outcome, ".1")
    z2 <- paste0(outcome, ".2")
    
    if(length(control.personvars) > 0){cvp1 <- paste0(control.personvars, ".1")} else {cvp1 <- c()}
    if(length(control.personvars) > 0){cvp2 <- paste0(control.personvars, ".2")} else {cvp2 <- c()}
    
    cvc <- control.couplevars
    
    if(length(control.redundant) > 0){cvg1 <- paste0(control.redundant, ".1")} else {cvg1 <- c()}
    if(length(control.redundant) > 0){cvg2 <- paste0(control.redundant, ".2")} else {cvg2 <- c()}
    
 
    # grand mean centering 
    
    gm <- mean( c(dat[, x1], dat[, x2]) , na.rm=T )
    
    x1c <- paste0(x1, ".c")
    dat[, x1c] <- dat[, x1] - gm
    
    x2c <- paste0(x2, ".c")
    dat[, x2c] <- dat[, x2] - gm


    # add second-order variables 

    x1c.sq <- paste0(x1c, "_squared")
    dat[, x1c.sq] <- dat[, x1c]^2
    
    x1c.x2c <- paste0(x1c, "_times_", x2c)
    dat[, x1c.x2c] <- dat[, x1c]*dat[, x2c]
    
    x2c.sq <- paste0(x2c, "_squared")
    dat[, x2c.sq] <- dat[, x2c]^2
    
    
    
  # prepare handling of control variables
    
    # cv flag if control variables are involved
    is.cv <- (length(control.personvars) > 0) | (length(control.couplevars) > 0) | (length(control.redundant) > 0)
    
    ## collect info
    if(is.cv){
      cv <- data.frame(
        type = c( rep("cvc", length(cvc)), rep("cvp", length(cvp1)), rep("cvg", length(cvg1)) ), 
        vars_1 = c(cvc, cvp1, cvg1),
        vars_2 = c(cvc, cvp2, cvg1), # same "redundant" control variables as in equation 1
        coefs_1 = paste0("b", 5+(1:length(c(cvc, cvp1, cvg1))), "_1"),
        coefs_2 = paste0("b", 5+(1:length(c(cvc, cvp1, cvg1))), "_2")
      )
    } else {
      cv <- NULL
    }
           

    
  # detection and treatment of influential cases (separate for the two partners)
    
    # detect influential cases and warn

    # partner 1 model
    
      # define control variable part of the regression model
      if(is.cv){
        cv_lm_1 <- paste("+", paste(cv$vars_1, collapse = " + "))
      } else {
        cv_lm_1 <- ""
      }

      # run polynomial regressions as OLS linear models
      model_1 <- paste0(z1, " ~ ", paste("1", x1c, x2c, x1c.sq, x1c.x2c, x2c.sq, sep=" + "), cv_lm_1)
      lm_1 <- lm(model_1, data=dat, na.action=na.exclude)
  
      # identify and save positions of outliers
      dat$out_1 <- FALSE
      inf_1 <- influence.measures(lm_1)
      dat$out_1 <- apply(inf_1$is.inf[, c("dffit", "cook.d", "hat")], 1, sum) == 3
  
      n.out_1 <- sum(na.omit(dat$out_1) == TRUE)
      
      if (n.out_1 > 0 & !rm.influential) {
        warning(paste("There are", n.out_1, "influential case(s) for the partner 1 model. 
                      I did not remove any of them but you might want to run a sensitivity analysis setting rm.influential=TRUE. 
                      The influential cases are in row(s):", paste(which(dat$out_1 == TRUE) , collapse=", ")))
      }
  
    
    # partner 2 model
      
      # define control variable part of the regression model
      if(is.cv){
        cv_lm_2 <- paste("+", paste(cv$vars_2, collapse = " + "))
      } else {
        cv_lm_2 <- ""
      }
      
      
      # run polynomial regressions as OLS linear models
      model_2 <- paste0(z2, " ~ ", paste("1", x1c, x2c, x1c.sq, x1c.x2c, x2c.sq, sep=" + "), cv_lm_2)
      lm_2 <- lm(model_2, data=dat, na.action=na.exclude)
      
      # identify and save positions of outliers
      dat$out_2 <- FALSE
      inf_2 <- influence.measures(lm_2)
      dat$out_2 <- apply(inf_2$is.inf[, c("dffit", "cook.d", "hat")], 1, sum) == 3
      
      n.out_2 <- sum(na.omit(dat$out_2) == TRUE)
      
      if (n.out_2 > 0 & !rm.influential) {
        warning(paste("There are", n.out_2, "influential case(s) for the partner 2 model. 
                      I did not remove any of them but you might want to run a sensitivity analysis setting rm.influential=TRUE. 
                      The influential cases are in row(s):", paste(which(dat$out_2 == TRUE) , collapse=", ")))
      }  
      

    # remove all influential cases if requested (set all variables to NA for these cases)
      
      dat$out <- dat$out_1 | dat$out_2
      
      if(rm.influential){

        n.out <- sum(na.omit(dat$out) == TRUE)
        
        # keep cases with missing value on data$out (missings will be replaced with missing option)
        dat$out[is.na(dat$out)] <- FALSE
        
        dat[dat$out==TRUE, names(dat)[! names(dat) %in% c(couple.identifier, "out_1", "out_2", "out")] ] <- NA
        
        warning(paste("Removed", n.out, "influential case(s). They are in row(s):", paste(which(dat$out == TRUE) , collapse=", ")))
        }
    

      
  # define equations for the two rows of the DRSA model
      
      # construct control variable parts of the model
      
      ## cv parts of the model equation
      if(is.cv) {
        cv_poly_1 <- paste0(" + ", paste(paste(cv$coefs_1, "*", cv$vars_1), collapse=" + ") )
        cv_poly_2 <- paste0(" + ", paste(paste(cv$coefs_2, "*", cv$vars_2), collapse=" + ") )
        } else {
        cv_poly_1 <- ""
        cv_poly_2 <- ""
        }
      
      ## cv coefficient constraints         
      if(is.cv) {
        cv[cv$type=="cvg", "coefs_1"] <- paste0("-", cv[cv$type=="cvg", "coefs_1"])
        cv_constraints <- paste( paste(cv$coefs_2, "==", cv$coefs_1), collapse=" \n " )
      } else {
        cv_constraints <- ""
      }
      
      
      # polynomial models

      poly_1 <- paste0(z1, " ~ b0_1*1 + b1_1*", x1c, " + b2_1*", x2c, " + b3_1*", x1c.sq, " + b4_1*", x1c.x2c, " + b5_1*", x2c.sq, cv_poly_1)
      poly_2 <- paste0(z2, " ~ b0_2*1 + b1_2*", x2c, " + b2_2*", x1c, " + b3_2*", x2c.sq, " + b4_2*", x1c.x2c, " + b5_2*", x1c.sq, cv_poly_2)
      

  
  # fit model: full DRSA model
    
    model_drsa <- paste(poly_1, poly_2,
                        
                        paste0(z1, " ~~ ", z2), 
                        
                        paste0(z1, " ~~ evar1*", z1), # label error variance
                        paste0(z2, " ~~ evar2*", z2), # label error variance
                        
                        "b0_1 == b0_2",
                        "b1_1 == b1_2",
                        "b2_1 == b2_2",
                        "b3_1 == b3_2",
                        "b4_1 == b4_2",
                        "b5_1 == b5_2",
                        
                        cv_constraints, 
                        
                        "evar1 == evar2", 

                        "a1_1 := b1_1 + b2_1",
                        "a2_1 := b3_1 + b4_1 + b5_1",
                        "a3_1 := b1_1 - b2_1",
                        "a4_1 := b3_1 - b4_1 + b5_1",
                        "a5_1 := b3_1 - b5_1",
                        
                        "a1_2 := b1_2 + b2_2",
                        "a2_2 := b3_2 + b4_2 + b5_2",
                        "a3_2 := b1_2 - b2_2",
                        "a4_2 := b3_2 - b4_2 + b5_2",
                        "a5_2 := b3_2 - b5_2", 

                        sep="\n")
 
    set.seed(seed) # ensure that the results of bootstrapped standard errors are reproducible
    
    sem_drsa <- sem(model_drsa, data=dat, fixed.x=TRUE, missing=missing, se=se, bootstrap=bootstrap) 
     
    

  ## build and return result object
  
    res <- list(sem_drsa = sem_drsa, 

                data_used = dat,

                poly_1 = poly_1,
                poly_2 = poly_2, 
                
                arguments = match.call()

                )
  
    return(res)
  
}



