## This code is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 2 of the License, or
## (at your option) any later version.
## 
## For a copy of the GNU General Public License, 
## see <http://www.gnu.org/licenses/>.

## (c) 2021 Yoobin Park, Sarah Humberg



# .................................................... ----
# DESCRIPTION                                          ----


# This code is the main analysis code that accompanies the preregistration for the following study: 
# 
# Title: Does Grateful Love Take Two? New insights into Partner Match on Gratitude Using Seven Longitudinal Datasets
# 
# Authors: Yoobin Park, Sarah Humberg, Amie M. Gordon, Amy Muise, Emily A. Impett


# This code uses fake data to demonstrate the planned analyses. 
#
# The six fake data sets roughly mimic the data from the six samples that will be used in this study:
#
# Sample 1: Self-Expansion Study (satisfaction assessed with the PRQC, commitment: PRQC)
# Sample 2: Sex and Touch study (satisfaction: PRQC, commitment: PRQC)
# Sample 3: Sexual Decision Making Study (satisfaction: IMS, commitment: IMS)
# Sample 4: Toronto Couples Study (satisfaction: CSI, commitment: not assessed)
# Sample 5: SIR Lab Interaction Study (satisfaction: CSI, commitment: IMS)
# Sample 6: Sleep and Relationships Study (satisfaction: CSI, commitment: IMS)


# The fake data sets include the following variables:
#
# sample = sample identifier
# coupleID = couple identifier; within each sample, it is unique for each couple
# personID = person identifier (will not be used)
# 
# gender = participants' gender (0 = man, 1 = woman)
# age = participants' age
#
# rlength = relationship length (couple-level variable)
# 
# grat.t1 = level of gratitude, measured at time point 1
# grat.t2 = level of gratitude, measured at time point 2
# 
# sat.prqc.t1 = relationship satisfaction at time point 1, measured with the PRQC (only in samples 1 and 2)
# sat.prqc.t2 = relationship satisfaction at time point 2, measured with the PRQC (only in samples 1 and 2)
#
# sat.ims.t1 = relationship satisfaction at time point 1, measured with the IMS (only in sample 3, will not be used)
# sat.ims.t2 = relationship satisfaction at time point 2, measured with the IMS (only in sample 3, will not be used)
#
# sat.csi.t1 = relationship satisfaction at time point 1, measured with the CSI (only in samples 4 to 6)
# sat.csi.t2 = relationship satisfaction at time point 2, measured with the CSI (only in samples 4 to 6)




# .................................................... ----
# PREPARATIONS                                         ----


# source helpers file that loads the relevant packages and self-defined functions
source("code1_helpers.R")


# read prepared data
sample1 <- read.table("fakedata_sample1.txt", header = TRUE, sep = ";")
sample2 <- read.table("fakedata_sample2.txt", header = TRUE, sep = ";")
sample3 <- read.table("fakedata_sample3.txt", header = TRUE, sep = ";")
sample4 <- read.table("fakedata_sample4.txt", header = TRUE, sep = ";")
sample5 <- read.table("fakedata_sample5.txt", header = TRUE, sep = ";")
sample6 <- read.table("fakedata_sample6.txt", header = TRUE, sep = ";")


# pool the data
df <- plyr::rbind.fill(sample1, sample2, sample3, sample4, sample5, sample6)


# check whether each couple has a unique ID in the pooled data (if not, this must be solved before proceeding)
length(unique(df$coupleID)) == .5*nrow(df)


# generate dummy variables that encode study membership
df <- fastDummies::dummy_cols(df, select_columns="sample")



# .................................................... ----
# MAIN ANALYSES                                        ----


## Research Question 1 ----

# Are couples more satisfied with their relationship when the two partners are
# similarly high in their levels of gratitude towards each other, as compared to
# couples whose members have more discrepant levels of gratitude?

# outcome = satisfaction at t1
# predictors in RSA = gratitude at t1
# couple-level control variables: sample dummies


### Analysis 1a: Analysis for samples that used the PRQC as a measure of satisfaction (samples 1 and 2) ----


# pre-analysis: percentages of couples with congruent (within 0.5 SD) versus incongruent (discrepant by more than 0.5 SD) predictor values
discr.long(data = df[df$sample %in% c(1,2), ], couple.identifier = "coupleID", x = "grat.t1")


# DRSA
drsa_sat.prqc.t1 <- drsa(data = df[df$sample %in% c(1,2), ],
                         couple.identifier = "coupleID",
                         predictor = "grat.t1",
                         outcome = "sat.prqc.t1", 
                         control.couplevars = c("sample_1"),
                         se = "boot"                          
                         )


# inspect results
parameterEstimates(drsa_sat.prqc.t1$sem_drsa)




### Analysis 1b: Analysis for samples that used the CSI as a measure of satisfaction (samples 4, 5, 6) ----


# pre-analysis: percentages of couples with congruent (within 0.5 SD) versus incongruent (discrepant by more than 0.5 SD) predictor values
discr.long(data = df[df$sample %in% c(4,5,6), ], couple.identifier = "coupleID", x = "grat.t1")


# DRSA
drsa_sat.csi.t1 <- drsa(data = df[df$sample %in% c(4,5,6), ],     
                        couple.identifier = "coupleID",
                        predictor = "grat.t1",
                        outcome = "sat.csi.t1", 
                        control.couplevars = c("sample_4","sample_5"),
                        se = "boot"
                        )

parameterEstimates(drsa_sat.csi.t1$sem_drsa)



## Research Question 2 ----

# Do couples increase in satisfaction when the two partners are similarly high
# in their levels of gratitude towards each other at baseline?

# outcome = satisfaction at t2
# predictors in RSA = gratitude at t1
# couple-level control variables: sample dummies
# person-level control variables: satisfaction at t1


### Analysis 2a: Analysis for samples that used the PRQC as a measure of satisfaction (samples 1 and 2) ----


# pre-analysis: see the pre-analysis for Analysis 1a


# DRSA
drsa_sat.prqc.t2_control_sat.t1 <- drsa(data = df[df$sample %in% c(1,2), ],
                                        couple.identifier = "coupleID",
                                        predictor = "grat.t1",
                                        outcome = "sat.prqc.t2", 
                                        control.couplevars = c("sample_1"), 
                                        control.personvars = c("sat.prqc.t1"),
                                        se = "boot"
                                        )

parameterEstimates(drsa_sat.prqc.t2_control_sat.t1$sem_drsa)



### Analysis 2b: Analysis for samples that used the CSI as a measure of satisfaction (samples 4, 5, 6) ----


# pre-analysis: see the pre-analysis for Analysis 1b


# DRSA
drsa_sat.csi.t2_control_sat.t1 <- drsa(data = df[df$sample %in% c(4,5,6), ],
                                    couple.identifier = "coupleID",
                                    predictor = "grat.t1",
                                    outcome = "sat.csi.t2", 
                                    control.couplevars = c("sample_4","sample_5") , 
                                    control.personvars = c("sat.csi.t1"),
                                    se = "boot"
)

parameterEstimates(drsa_sat.csi.t2_control_sat.t1$sem_drsa)




## Research Question 3 ----

# Are couples more committed to their relationship when the two partners are
# similarly high (vs. discrepant) in their levels of gratitude towards each other?

# outcome = commitment at t1
# predictors in RSA = gratitude at t1
# couple-level control variables: sample dummies

# -> implementation just as for RQ 1, but with commitment instead of satisfaction 

# Analysis 3a will use data from samples in which commitment was assessed with the PRQC = samples 1 and 2
# Analysis 3b will use data from samples in which commitment was assessed with the IMS = samples 3, 5, and 6




## Research Question 4 ----

# Do couples increase in commitment when the two partners are similarly high in
# their levels of gratitude towards each other at baseline?

# outcome = commitment at t2
# predictors in RSA = gratitude at t1
# couple-level control variables: sample dummies
# person-level control variables: commitment at t1

# -> implementation just as for RQ 2, but with commitment instead of satisfaction

# Analysis 4a will use data from samples in which commitment was assessed with the PRQC = samples 1 and 2
# Analysis 4b will use data from samples in which commitment was assessed with the IMS = samples 3, 5, 6




## ........................
## Research Question 5 ----

# Do couples report higher levels of gratitude at the follow-up when the two partners 
# are similarly high in their levels of gratitude towards each other at baseline?

# outcome = gratitude at t2
# predictors in RSA = gratitude at t1
# couple-level control variables: sample dummies


# pre-analysis: percentages of couples with congruent (within 0.5 SD) versus incongruent (discrepant by more than 0.5 SD) predictor values
discr.long(data = df, couple.identifier = "coupleID", x = "grat.t1")


# DRSA
drsa_grat.t2 <- drsa(data = df,
                     couple.identifier = "coupleID",
                     predictor = "grat.t1",
                     outcome = "grat.t2", 
                     control.couplevars = c("sample_1","sample_2","sample_3","sample_4","sample_5"),
                     se = "boot"
                     )

parameterEstimates(drsa_grat.t2$sem_drsa)




# .................................................... ----
# SENSITIVITY ANALYSES                                 ----


## ..............................................
## Sensitivity Analysis 1: Control variables ----

# Re-run the DRSA analyses with control for gender, age, and relationship length.
# For example:


### Sensitivity Analysis 1 for Analysis 1a ----

# outcome = satisfaction at t1
# predictors in RSA = gratitude at t1
# couple-level control variables: sample dummies
# person-level control variables: gender, age, relationship length 

drsa_sat.prqc.t1_control.age.gender.rlength <- drsa(data = df[df$sample %in% c(1,2), ],
                                                    couple.identifier = "coupleID",
                                                    predictor = "grat.t1",
                                                    outcome = "sat.prqc.t1", 
                                                    control.couplevars = c("sample_1", "rlength"),
                                                    control.personvars = c("age", "gender"),
                                                    se = "boot"
                                                    )

parameterEstimates(drsa_sat.prqc.t1_control.age.gender.rlength$sem_drsa)



# . . . and accordingly for all other analyses.




## ..............................................
## Sensitivity Analysis 2: Influential cases ----

# In case that influential cases were identified in any of the main analyses, we will re-run these analyses with data that exlude the influential cases to see whether conclusions change.

# For example:


### Sensitivity Analysis 2 for Analysis 1a ----

# were there any influential cases?

any( drsa_sat.prqc.t1$data_used$out == TRUE )

# -> that's not the case for the simulated data. 


# If influential cases *were* detected, we would re-run Analysis 1a with exclusion of influential cases and see whether conclusions change:

drsa_sat.prqc.t1_exl.inf <- drsa(data = df[df$sample %in% c(1,2), ],    
                                 couple.identifier = "coupleID",
                                 predictor = "grat.t1",
                                 outcome = "sat.prqc.t1", 
                                 control.couplevars = c("sample_1"),
                                 se = "boot", 
                                 rm.influential = TRUE
)

parameterEstimates(drsa_sat.prqc.t1_exl.inf$sem_drsa)



# . . . and accordingly for all other analyses.



